#ifndef CoCoA_CpuTimeLimit_H
#define CoCoA_CpuTimeLimit_H

//   Copyright (c)  2017  John Abbott, Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.
//   -------------------------------------------------------------------------


#include "CoCoA/interrupt.H"

#include <iosfwd>
//using std::ostream;

namespace CoCoA
{

  class InterruptedByTimeout: public InterruptReceived
  {
  public:
    InterruptedByTimeout(const char* const context): InterruptReceived(context) {}
//    InterruptedByTimeout(const std::string& context): InterruptReceived(context) {}
    virtual ~InterruptedByTimeout();
    virtual std::ostream& myOutputSelf(std::ostream& out) const;
  private: // same data members as class InterruptReceived
  };


  class CpuTimeLimit
  {
  public:
    explicit CpuTimeLimit(double t);
    ~CpuTimeLimit();
    std::ostream& myOutputSelf(std::ostream& out) const;
  private: // disable copy ctor and assignment
    CpuTimeLimit(const CpuTimeLimit&); // disabled -- never defined
    CpuTimeLimit& operator=(const CpuTimeLimit&); // disabled -- never defined
  private:
    friend void deactivate(CpuTimeLimit& CpuLimit) { CpuLimit.myDeactivate(); }
    void myDeactivate();
  private: // data members
    double myStartTime;
    double myExpirationTime; // just for printing
    double myPrevExpirationTime; // read the documentation!!
  };


  void CheckForTimeout(const char* const context);  // throws if timeout has occured
//  void CheckForTimeout(const std::string& context); // throws if timeout has occured
  std::ostream& operator<<(std::ostream& out, const CpuTimeLimit& CpuLimit);
  

} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/CpuTimeLimit.H,v 1.4 2017/07/22 12:56:34 abbott Exp $
// $Log: CpuTimeLimit.H,v $
// Revision 1.4  2017/07/22 12:56:34  abbott
// Summary: Updated exception type
//
// Revision 1.3  2017/07/21 15:06:09  abbott
// Summary: Major revision -- no longer needs BOOST
//
// Revision 1.2  2017/07/21 13:21:21  abbott
// Summary: Split olf interrupt into two ==> new file SignalWatcher; refactored interrupt and CpuTimeLimit
//
// Revision 1.1  2017/07/15 15:18:43  abbott
// Summary: Added CpuTimeLimit
//
//

#endif
